""" Domain Filter to API params"""
from isahitlab.domain.task import TaskFilters

def task_filters_mapper(filters: TaskFilters):
    params = {
        "pagination" : "false" if filters.pagination == False else "true",
        "page" : filters.page,
        "limit" : filters.limit,
        "projectId" : filters.project_id,
    }

    if filters.batch_id_in and len(filters.batch_id_in):
        params['batchIds'] = filters.batch_id_in

    if filters.status_in and len(filters.status_in):
        params['statuses'] = filters.status_in

    if filters.name_like and len(filters.name_like):
        params['nameLike'] = [filters.name_like]

    if filters.task_id_in and len(filters.task_id_in):
        params['taskIds'] = [filters.task_id_in]

    if filters.name_in and len(filters.name_in):
        params['taskNames'] = [filters.name_in]

    if filters.optional_fields and len(filters.optional_fields):
        params['optionalFields'] = filters.optional_fields

    if filters.created_at_gt and len(filters.created_at_gt):
        if "createdAt" not in params:
            params["createdAt"] = []
        params['createdAt'].append(">{}".format(filters.created_at_gt))

    if filters.created_at_gte and len(filters.created_at_gte):
        if "createdAt" not in params:
            params["createdAt"] = []
        params['createdAt'].append(">={}".format(filters.created_at_gte))

    if filters.created_at_lt and len(filters.created_at_lt):
        if "createdAt" not in params:
            params["createdAt"] = []
        params['createdAt'].append("<{}".format(filters.created_at_lt))

    if filters.created_at_lte and len(filters.created_at_lte):
        if "createdAt" not in params:
            params["createdAt"] = []
        params['createdAt'].append("<={}".format(filters.created_at_lte))

    if filters.updated_at_gt and len(filters.updated_at_gt):
        if "updatedAt" not in params:
            params["updatedAt"] = []
        params['updatedAt'].append(">{}".format(filters.updated_at_gt))

    if filters.updated_at_gte and len(filters.updated_at_gte):
        if "updatedAt" not in params:
            params["updatedAt"] = []
        params['updatedAt'].append(">={}".format(filters.updated_at_gte))

    if filters.updated_at_lt and len(filters.updated_at_lt):
        if "updatedAt" not in params:
            params["updatedAt"] = []
        params['updatedAt'].append("<{}".format(filters.updated_at_lt))

    if filters.updated_at_lte and len(filters.updated_at_lte):
        if "updatedAt" not in params:
            params["updatedAt"] = []
        params['updatedAt'].append("<={}".format(filters.updated_at_lte))

    if filters.data:
        for field in filters.data:
            params[f'data.{field}'] = filters.data[field]

    if filters.properties:
        for property in filters.properties:
            params[f'properties.{property}'] = filters.properties[property]

    return params


from dataclasses import dataclass
from typing import Any, Dict, List, Literal, NewType, Optional, Union

from .batch import BatchId
from .pagination import PaginationFilters
from .project import ProjectId

TaskId = NewType("TaskId", str)


TaskStatus = Literal["pending", "complete",
                     "to-review", "reviewed", "configuring"]
TaskOptionalFields = Literal["metrics", "jobs", "data", "data.mask"]
TaskCompatibilityMode = Literal["kili", "lab"]


@dataclass
class TaskPayload(dict):
    """Task payload for task creation


    Args:
        name: Name of the task (should be unique in a batch)
        resources: Path or URL of a resource (ex.: "./images/image1.jpg", "https:://domain.com/image1.jpg")
        data: Initial data for the task (depends on project type and configuration)
        properties: See `update_properties_of_tasks(...)`
    """
    # Inherit from dict to make it JSON serializable

    name: str
    data: Dict[str, Any]
    resources: Optional[List[str]] = None
    properties: Optional[Dict[str, Any]] = None

    def __init__(self, name: str, resources: Optional[List[str]], data:  Dict[str, Any], properties: Optional[Dict[str, Any]]):
        self.name = name
        self.resources = resources
        self.data = data
        self.properties = properties
        dict.__init__(self, name=name, resources=resources, data=data)


@dataclass
class TaskFilters(PaginationFilters):
    """Task filters for running a task search."""

    batch_id_in: Optional[List[BatchId]] = None
    project_id: Optional[ProjectId] = None
    status_in: Optional[List[TaskStatus]] = None
    task_id_in: Optional[List[TaskId]] = None
    name_in: Optional[List[str]] = None
    name_like: Optional[str] = None
    created_at_gt: Optional[str] = None
    created_at_gte: Optional[str] = None
    created_at_lt: Optional[str] = None
    created_at_lte: Optional[str] = None
    updated_at_gt: Optional[str] = None
    updated_at_gte: Optional[str] = None
    updated_at_lt: Optional[str] = None
    updated_at_lte: Optional[str] = None
    optional_fields: Optional[List[TaskOptionalFields]] = None
    data: Optional[Dict[str, Union[str, int, float]]] = None
    properties: Optional[Dict[str, Union[str, int, float]]] = None



from typing import Dict, Generator, Iterable, List, Optional, Union, cast

from isahitlab.actions.base import BaseAction
from isahitlab.domain.integration import (IntegrationVisibility, IntegrationType, IntegrationFilters, IntegrationPayload)
from isahitlab.operations.integration.get_integrations import GetIntegrationsOperation
from isahitlab.operations.integration.create_integration import CreateIntegrationOperation
from typeguard import typechecked


class IntegrationsActions(BaseAction):
    """Integrations actions"""

    @typechecked
    def integrations(
        self,
        visibility_in: Optional[List[IntegrationVisibility]] = None,
        search: Optional[str] = None,
        disable_progress_bar: Optional[bool] = False,
        iterate: Optional[bool] = False
    ) -> Union[Generator[Iterable[Dict],None,None],Iterable[Dict]] :
        """ Get the list of your integrations and those of your organization
        
        Args:
            visibility_in: Only in those visibilities.
                Possible choices: `organization`, `private`.
            search: Quicksearch,
            disable_progress_bar: Disable the progress bar display,
            iterate: Return a generator

        Returns:
            List (or Generator) of integration representations
        """

        filters = IntegrationFilters(
            visibility_in = cast(List[IntegrationVisibility], visibility_in) if visibility_in else None,
            search = search if search else None,
        )
        
        operation_gen = GetIntegrationsOperation(self.http_client).run(filters=filters, disable_progress_bar=disable_progress_bar)

        if iterate:
            return operation_gen
        else:
            return list(operation_gen)
    

    @typechecked
    def create_integration(
        self,
        name: str,
        type: IntegrationType,
        visibility: IntegrationVisibility,
        bucket_access_point: str,
        role_id: Optional[str] = None,
        external_id: Optional[str] = None,
        disable_progress_bar: Optional[bool] = False
    ) -> Dict:
        """ Create an integration

        Args:
            name: Name of the batch
            type: Type of the integration,
                Possible choices: `GCP`, `S3`.
            visibility: Visibility of the integration
                Possible choices: `organization`, `private`.
            bucket_access_point: Access point for S3 or bucket for GCP
            role_id: Role ID or ARN for S3 integration
            external_id: Client external ID for S3 integration
            disable_progress_bar: Disable the progress bar display

        """

        integration = IntegrationPayload(
            name=name,
            type=type,
            visibility=visibility,
            access_point=bucket_access_point,
            role_id=role_id,
            external_id=external_id
        )

        if integration.type == 'S3' and (not integration.role_id or not integration.external_id):
            raise ValueError('role_id and external_id are mandatory for S3 integration')


        return CreateIntegrationOperation(self.http_client).run(integration=integration, disable_progress_bar=disable_progress_bar)



import logging
from typing import Dict, Iterable, Optional

from isahitlab.api.batch.api import BatchApi
from isahitlab.api.workflow.api import WorkflowApi
from isahitlab.domain.batch import BatchFilters, BatchId
from isahitlab.domain.project import ProjectId
from isahitlab.domain.workflow import WebhookPayload, WorkflowId
from isahitlab.operations.base import BaseAction
from tqdm import tqdm
from typeguard import typechecked

logger = logging.getLogger('isahitlab.get_webhook')


class GetWebhook(BaseAction):
    """Create task actions
    """


    _batch_api : BatchApi
    _workflow_api : WorkflowApi

    @typechecked
    def run(
        self,
        project_id: ProjectId,
        batch_id: Optional[BatchId] = None,
        workflow_id: Optional[WorkflowId] = None

    ) -> Dict:
        """ Setup a webhook for a workflow

        !!! warning
            You must provide either a batch_id or a workflow_id
        
        Args:
            project_id: ID of the project
            batch_id: ID of the batch 
            workflow_id: ID of the workflow project
            webhook : See WebhookPayload (url : str, header : str)
            disable_progress_bar: Disable the progress bar display

        """
        self._batch_api = BatchApi(self._http_client)
        self._workflow_api = WorkflowApi(self._http_client)

        if not workflow_id and not batch_id:
            raise ValueError('You must provide either a batch_id or a workflow_id')
        
        if workflow_id:
            return self._workflow_api.get_workflow_by_id(workflow_id=workflow_id)
        elif batch_id:
            batch = self._batch_api.get_batch_by_id(batch_id=batch_id)
            if not batch:
                raise ValueError("Batch not found")
            if batch["projectId"] != project_id:
                raise ValueError("The batch is not part of the project")
            
            workflow_id = batch.get("workflow", {}).get("id")
            if not workflow_id:
                raise Exception("The batch doesn't have a workflow")

            workflow = self._workflow_api.get_workflow_by_id(workflow_id=workflow_id)

            if not workflow:
                raise Exception("Workflow not found")
            
            return workflow.get('webhook', None)


from tqdm import tqdm
from typing import Optional, Dict, List, Generator
from isahitlab.operations.base import BaseAction
from isahitlab.domain.integration import (IntegrationFilters)
from isahitlab.api.integration.api import IntegrationApi

from typeguard import typechecked

class GetIntegrationsOperation(BaseAction):
    """Get integrations actions"""

    @typechecked
    def run(
        self,
        filters: IntegrationFilters,
        disable_progress_bar: Optional[bool] = False
    ) -> Generator[List[Dict], None, None]:
        """ Get the your integrations
        
        Args:
            filters : IntegrationVisibility object
            disable_progress_bar: Disable the progress bar display
        """

        integration_api = IntegrationApi(self._http_client)
        with tqdm(total=1,  disable=disable_progress_bar, desc="Loading integrations... ") as loader:
             for (docs, loaded, total) in integration_api.get_all_integrations(filters):
                loader.total = total
                yield from docs
                loader.update(loaded - loader.n)
    


from typing import Dict, Generator, Iterable, List, Optional, Union, cast

from isahitlab.actions.base import BaseAction
from isahitlab.domain.workflow import WebhookPayload
from isahitlab.operations.webhook.get_webhook import GetWebhook
from isahitlab.operations.webhook.setup_webhook import SetupWebhook
from typeguard import typechecked


class WebhookActions(BaseAction):
    """Webhook actions"""

    @typechecked
    def setup_webhook(
        self,
        project_id: str,
        webhook_url: str,
        webhook_header: Optional[str] = None,
        batch_id: Optional[str] = None,
        workflow_id: Optional[str] = None,
        disable_progress_bar: Optional[bool] = False
    ) -> None :
        """ Setup a webhook for a workflow

        !!! warning
            If you only set the `project_id` parameter, the webhook will be set on all batches of the project
        
        Args:
            project_id: ID of the project
            webhook_url: URL to call
            webhook_header: Authorization header to set on the webhook requests
            batch_id: ID of the batch 
            workflow_id: ID of the workflow project
            disable_progress_bar: Disable the progress bar display

        Returns:
            None
        """

        webhook_payload : WebhookPayload = WebhookPayload(url=webhook_url, header=webhook_header)
        
        SetupWebhook(self.http_client).run(project_id=project_id, workflow_id=workflow_id, batch_id=batch_id, webhook=webhook_payload, disable_progress_bar=disable_progress_bar)

        print("Webhook setup complete!")

    @typechecked
    def get_webhook(
        self,
        project_id: str,
        batch_id: Optional[str] = None,
        workflow_id: Optional[str] = None
    ) -> Dict :
        """ Get a webhook for a workflow

        !!! warning
            You must provide a `batch_id` of a `workflow_id`
        
        Args:
            project_id: ID of the project
            batch_id: ID of the batch 
            workflow_id: ID of the workflow project

        Returns:
            Webhook representation
        """

        if not batch_id and not workflow_id:
            raise ValueError("You must provide a batch ID of a workflow ID")

        
        return GetWebhook(self.http_client).run(project_id=project_id, workflow_id=workflow_id, batch_id=batch_id)


from tqdm import tqdm
from typing import Optional, Dict, List, Generator
from isahitlab.operations.base import BaseAction
from isahitlab.domain.project import ProjectFilters
from isahitlab.api.project.api import ProjectApi

from typeguard import typechecked

class GetProjectsOperation(BaseAction):
    """Get project actions"""

    @typechecked
    def run(
        self,
        filters: ProjectFilters,
        disable_progress_bar: Optional[bool] = False
    ) -> Generator[List[Dict], None, None]:
        """ Get projects
        
        Args:
            filters : ProjectFilters object
            disable_progress_bar: Disable the progress bar display
        """

        project_api = ProjectApi(self._http_client)
        with tqdm(total=1,  disable=disable_progress_bar, desc="Loading projects... ") as loader:
             for (docs, loaded, total) in project_api.get_all_projects(filters):
                loader.total = total
                yield from docs
                loader.update(loaded - loader.n)
    


import logging
import pathlib
import shutil
import tempfile
from datetime import datetime
from typing import Any, Dict, List

from isahitlab.helpers.labels import (
  extract_main_labels, flatten_available_labels_used_in_main_lists)
from isahitlab.helpers.path import check_extension
from isahitlab.helpers.points import (bbox_to_center_and_size,
                                      normalize_points, points_to_vertices,
                                      vertices_to_points, vertices_to_bbox)

from .base import BaseFormatter

logger = logging.getLogger(__name__)

class LabToYoloFormatter(BaseFormatter):
    """Formatter to export tasks to Yolo (v8) format 
    
    **archive.zip**

    - data.yaml
    - labels/
        - image1.jpg.txt
        - image2.jpg.txt
    
    **data.yaml**
    
    nc: 2
    names: ['label_1', 'label_2']
    
    **image1.jpg.txt**

    0 0.6545064391681276 0.4235482071561457 0.0797404071439336 0.18190535356334903
    0 0.3622486209742762 0.3927660911752291 0.07606270256677716 0.15973167539023203
    1 0.34460042208424047 0.5709931135225523 0.06213055937206563 0.1554331519686042
    2 0.3407572947004013 0.49469991805078334 0.028823455378793328 0.03700789332585808
    3 0.39520159930478876 0.5274376698390424 0.025620849225594045 0.04099335876095056
    3 0.38068257272515843 0.47885714285714287 0.060650859736031015 0.09142857142857141

    **For rectangle:**     
    <class> <x_center> <y_center> <width> <height>

    **For polygon:**
    <class> <x> <y> <x> <y> <x> <y> ...  <x> <y>

    **For keypoints:**
    <class> <x_center> <y_center> <width> <height> <px1> <py1> <px2> <py2> ... <pxn> <pyn>

    https://yolov8.org/yolov8-label-format/

    """
    classes : List[Dict[str, Any]]= None
    temp_dir : tempfile.TemporaryDirectory = None
    is_segmentation: bool = False

    def __init__(self, project_configuration : Dict, options : Dict = {}) -> None :
        
        if options.get("in_memory", None):
            raise Exception('"in_memory" not supported')
        
        self._init_configuration(project_configuration, options, {
            "compatible_types" : [                
                "form.tool-iat-rectangle", 
                "iat-rectangle",
                "form.tool-iat-polygon", 
                "iat-polygon",
                "form.tool-iat-polyline", 
                "iat-polyline",
                "form.tool-iat-graph", 
                "iat-graph",
                "form.tool-iat-segmentation", 
                "iat-segmentation"
            ]
        })

        # Check if is sequence
        if self._project_configuration.get("metadata", {}).get("toolOptions", {}).get("sequence", False):
            raise Exception('LabToYoloFormatter does not support sequences')

        # Check output filename
        if options.get("output_filename", None) and not check_extension(options.get("output_filename", ""), ".zip"):
            raise Exception('"output_filename" must have .zip extension')

        self.is_segmentation = self._type == "iat-segmentation" or self._type == "tool-iat-segmentation"
    
    ### Transform

    def format_tasks(self, tasks: List[Dict]) -> List[Dict]:
        formatted_tasks = []

        for task in tasks:
            formatted_task = self._format_task(task)
            formatted_tasks.append(formatted_task)

        return formatted_tasks
    
    ### Export

    def _init_export(self):
        self.classes = flatten_available_labels_used_in_main_lists(self._project_configuration)
        self.temp_dir = tempfile.TemporaryDirectory()

    def append_task_to_export(self, task: Dict):
        if self.temp_dir == None:
            self._init_export()

        rows = self._format_task(task)
        task_name = task["name"]
        self._write_labels_file(task_name, rows)

    def complete_export(self):
        if self.temp_dir == None:
            logger.warning('Nothing to export')
            return
        
        self._write_classes_file()
        now_str = datetime.now().strftime("%Y-%m-%d_%H:%M:%S")

        default_folder = '.'
        extra_filename_part = "_{}".format(self._options.get("extra_filename_part")) if "extra_filename_part" in self._options else ""
        default_filename = f'lab_yolo{extra_filename_part}_{now_str}.zip'
        output_folder = self._options.get("output_folder", None) or default_folder
        outputfile_path = "{}/{}".format(output_folder, self._options.get("output_filename", None) or default_filename)

        self._make_archive(outputfile_path)

        self.temp_dir.cleanup()

        print("Exported to \"{}\"".format(str(pathlib.Path(outputfile_path).absolute())))

    def _format_task(self, task : Dict) -> Dict:
        task = self._normalize_task(task)
        rows = []
        annotations = task.get("data", {}).get("body", {}).get("annotations", [])
        image_size = task.get("data", {}).get("body", {}).get("image", None)
        
        for annotation in annotations:
            if self.is_segmentation:
                polygons = annotation.get('approximation',{}).get('polygons',[])
            else:
                polygons = annotation["polygons"]
            labels = annotation["labels"] if "labels" in annotation else {}
            main_labels = extract_main_labels(labels)
            #select first label
            label = main_labels[0] if len(main_labels) > 0 else None
            label_index = self.classes.index(label['id']) if label else -1
            
            for polygon in polygons:
                if self.is_segmentation and image_size: 
                    vertices = points_to_vertices(normalize_points(vertices_to_points(polygon["geometry"]["vertices"]), image_size))
                else:
                    vertices = polygon["geometry"]["vertices"]
                if polygon["geometry"]["type"] == "rectangle":
                    vertices = bbox_to_center_and_size(vertices)
                elif polygon["geometry"]["type"] == "graph":
                    bbox_vertices = bbox_to_center_and_size(vertices_to_bbox(vertices))
                    vertices = [*bbox_vertices, *vertices]
                rows.append([label_index, *vertices])

        return rows


    def _make_archive(self, output_filename: str) -> str:
        """Make the export archive."""
        path_folder = pathlib.Path(self.temp_dir.name)
        path_archive = shutil.make_archive(str(path_folder), "zip", path_folder)

        output_path = pathlib.Path(output_filename)
        output_path.parent.mkdir(parents=True, exist_ok=True)

        shutil.copy(path_archive, output_path)
        return str(output_path)


    def _write_classes_file(self):
        file_path = pathlib.Path(self.temp_dir.name) / "data.yaml"
        with file_path.open("wb") as fout:
            categories_str = ",".join([*map(lambda c : f"'{c}'", self.classes)])
            fout.write(f"nc: {len(self.classes)}\n".encode())
            fout.write(f"names: [{categories_str}]\n".encode())  # remove last comma

    def _write_labels_file(self, filename: str, rows : List[List]) -> None:
        file_path = pathlib.Path(self.temp_dir.name) / "labels" / f"{filename}.txt"
        file_path.parent.mkdir(parents=True, exist_ok=True)
        with file_path.open("wb") as fout:
            for row in rows:
                points_str = " ".join([str(i) for i in row])
                fout.write(f"{points_str}\n".encode())


    def prepare_import(self, tasks: List[Dict]):
        raise NotImplementedError


from typing import Dict
from isahitlab.api.base import BaseApi
from isahitlab.domain.project import ProjectId
from ..helpers import get_response_json, log_raise_for_status

class ProjectConfigurationApi(BaseApi):
    """Project configuration API Calls"""

    def get_project_configuration(self, project_id : ProjectId) -> Dict :
        """Get tasks"""

        project_configuration = self._http_client.get('api/project-configuration-manager/projects/{}/project-configuration'.format(project_id))

        log_raise_for_status(project_configuration)

        return get_response_json(project_configuration)



from typing import Dict, Generator, Iterable, List, Optional, Union, cast

from isahitlab.actions.base import BaseAction
from isahitlab.domain.project import (ProjectFilters, ProjectId, ProjectStatus,
                                      ProjectType)
from isahitlab.operations.project.get_projects import GetProjectsOperation
from typeguard import typechecked


class ProjectActions(BaseAction):
    """Projects actions"""

    @typechecked
    def projects(
        self,
        id_in: Optional[List[str]] = None,
        status_in: Optional[List[str]] = None,
        type_in: Optional[List[str]] = None,
        search: Optional[str] = None,
        disable_progress_bar: Optional[bool] = False,
        iterate: Optional[bool] = False
    ) -> Union[Generator[Iterable[Dict],None,None],Iterable[Dict]] :
        """ Get a projects list
        
        Args:
            id_in: ID of the projects
            status_in: Only in those statuses.
                Possible choices: `configuring`, `pending`, `complete`, `toBeDeleted`, `deleted`, `archived`.
            type_in: Only in those statuses.
                Possible choices:  `form`, `iat-rectangle`, `iat-polygon`, `iat-segmentation`, `iat-graph`, `iat-polyline`
            search: Quicksearch,
            disable_progress_bar: Disable the progress bar display,
            iterate: Return a generator

        Returns:
            List (or Generator) of project representations
        """

        filters = ProjectFilters(
            id_in=cast(List[ProjectId], id_in) if id_in else None, 
            status_in = cast(List[ProjectStatus], status_in) if status_in else None,
            type_in= cast(List[ProjectType], type_in) if type_in else None,
            search = search if search else None,
        )
        
        operation_gen = GetProjectsOperation(self.http_client).run(filters=filters, disable_progress_bar=disable_progress_bar)

        if iterate:
            return operation_gen
        else:
            return list(operation_gen)
    


# SPDX-FileCopyrightText: 2024-present Benjamin Piogé <benjamin@isahit.com>
#
# SPDX-License-Identifier: MIT

from typing import Any, Dict, List, Literal, Optional

from isahitlab.domain.project import ProjectType
from isahitlab.domain.task import TaskCompatibilityMode, TaskPayload
from jsonschema import Draft202012Validator, validate

from . import validation_schemas
from .form_validation_builder import build_validation_schema_for_form

ValidationSchema = Literal['lab_iat']


def validate_task_payloads(tasks: List[TaskPayload],  project_configuration: Dict[str, Any], compatibility_mode: Optional[TaskCompatibilityMode] = None) -> True:
    validation_schema = None

    project_type : ProjectType =  project_configuration['projectType']
    
    # Select schema
    if compatibility_mode == "kili":
        validation_schema = validation_schemas.KILI_SCHEMA        
    elif project_type == "iat-rectangle":
        validation_schema = validation_schemas.LAB_IAT_RECTANGLE_SCHEMA
    elif project_type == "iat-segmentation":
        validation_schema = validation_schemas.LAB_IAT_SEGMENTATION_SCHEMA
    elif project_type in ["iat-polygon", "iat-graph", "iat-polyline"]:
        validation_schema = validation_schemas.LAB_IAT_SCHEMA
    elif project_type == "form":
        validation_schema = build_validation_schema_for_form(project_configuration=project_configuration)
    

    if not validation_schema:
        raise Exception('Validation Schema not found for {}'.format(project_type))
    

    Draft202012Validator.check_schema(validation_schema)

    draft_202012_validator = Draft202012Validator(validation_schema)

    for task in tasks:
        if task.data:
            draft_202012_validator.validate(task.data)




"""isahit lab Python SDK client"""

import os
from typing import Optional

from . import log

logger = log.setup_custom_logger('IsahitLab')

from isahitlab.core.http.http_client import Credentials, HttpClient
from isahitlab.exceptions import AuthenticationFailed

from isahitlab.actions.task import TaskActions
from isahitlab.actions.dataset import DatasetActions
from isahitlab.actions.project_configuration import ProjectConfigurationActions
from isahitlab.actions.project import ProjectActions
from isahitlab.actions.webhook import WebhookActions
from isahitlab.actions.batch import BatchActions
from isahitlab.actions.integration import IntegrationsActions



class IsahitLab(
    TaskActions,
    ProjectConfigurationActions,
    DatasetActions,
    ProjectActions,
    WebhookActions,
    IntegrationsActions,
    BatchActions
):
    def __init__(
        self,
        access_id: Optional[str] = None,
        secret_key: Optional[str] = None,
        endpoint: Optional[str] = None,
        defer_auth: bool = True,
        http_retry_attempts = 0
    ) -> None:
        """Initialize isahit lab client

        Args:
            access_id: Use ISAHIT_LAB_API_ACCESS_ID environment variable if not provided
            secret_key: Use ISAHIT_LAB_API_SECRET_KEY environment variable if not provided
            endpoint: Use ISAHIT_LAB_API_ENDPOINT environment variable if not provided or the default https://hub-api.isahit.com/ 
        """
        
        endpoint = endpoint or os.getenv(
            "ISAHIT_LAB_API_ENDPOINT",
            "https://hub-api.isahit.com/"
        )
        
        access_id = access_id or os.getenv("ISAHIT_LAB_API_ACCESS_ID")
        secret_key = secret_key or os.getenv("ISAHIT_LAB_API_SECRET_KEY")
        
        if not access_id or not secret_key:
            raise AuthenticationFailed(access_id, secret_key)
        
        credentials: Credentials = {
            "access_id" : access_id,
            "secret_key" : secret_key
        }
        
        self.http_client = HttpClient(
            endpoint=endpoint,
            credentials=credentials,
            defer_auth=defer_auth,
            retry_max_attempts=http_retry_attempts
        )


from tqdm import tqdm
from typing import Optional, Dict
from isahitlab.operations.base import BaseAction
from isahitlab.domain.dataset import DatasetPayload
from isahitlab.api.dataset.api import DatasetApi

from typeguard import typechecked

class CreateDatasetOperation(BaseAction):
    """Create dataset operation"""

    @typechecked
    def run(
        self,
        project_id : str,
        dataset : DatasetPayload,
        disable_progress_bar: Optional[bool] = False
    ) -> Dict:
        """ Create a dataset
        
        Args:
            project_id: ID of the batch
            dataset : See DatasetPayload
            disable_progress_bar: Disable the progress bar display
        """

        dataset_api = DatasetApi(self._http_client)
        with tqdm(total=1,  disable=disable_progress_bar, desc="Creating dataset... ") as loader:
            created_dataset = dataset_api.create_dataset(project_id=project_id, dataset=dataset)
            loader.update(1)

        return created_dataset
    


from tqdm import tqdm
from typing import Optional, Dict
from isahitlab.operations.base import BaseAction
from isahitlab.domain.batch import BatchId
from isahitlab.api.batch.api import BatchApi

from typeguard import typechecked

class GetBatchOperation(BaseAction):
    """Get batch operation"""

    @typechecked
    def run(
        self,
        batch_id: BatchId,
        disable_progress_bar: Optional[bool] = False
    ) -> Dict:
        """ Get a batch by id
        
        Args:
            batch_id: ID of the batch
            disable_progress_bar: Disable the progress bar display
        """

        batch = None

        batch_api = BatchApi(self._http_client)
        with tqdm(total=1,  disable=disable_progress_bar, desc="Loading batch... ") as loader:
            batch = batch_api.get_batch_by_id(batch_id)
            loader.update(1)

        return batch
    


import json
import logging
from typing import Dict, Iterable, TypedDict, Union

import requests

from ..exceptions import (ApiBadRequest, ApiForbidden, ApiNotFound,
                          AuthenticationFailed, InternalServerError)


class Paginated(TypedDict):
    """Paginated result"""

    docs: Iterable[Dict]
    totalDocs: int
    limit: int
    totalPages: int
    page: int
    pagingCounter: int
    hasPrevPage: bool
    hasNextPage: bool
    prevPage: Union[int, None]
    nextPage: Union[int, None]


def log_raise_for_status(response: requests.Response) -> None:
    """Log the error message of a requests.Response if it is not ok.

    Args:
        response: a requests.Response
    """
    try:
        response.raise_for_status()
    except requests.exceptions.HTTPError as err:
        if err.response.status_code == 401:
            raise AuthenticationFailed()
        elif err.response.status_code == 400:
            raise ApiBadRequest(err.response.content)
        elif err.response.status_code == 404:
            raise ApiNotFound(err.response.content)
        elif err.response.status_code == 403:
            raise ApiForbidden(err.response.content)
        elif err.response.status_code == 500:
            raise InternalServerError(err.response.content)
        else:
            raise


def get_response_json(response: requests.Response) -> dict:
    """Get the json from a requests.Response.

    Args:
        response: a requests.Response
    """
    try:
        return response.json()
    except json.JSONDecodeError:
        logging.exception("An error occurred while decoding the json response")
        raise


from abc import ABC, abstractmethod
from typing import Dict, List

from isahitlab.helpers.inputs_layout import (IAT_INPUT_TYPES,
  extract_inputs_by_id_from_layout, extract_inputs_by_types_from_layout)


class BaseFormatter(ABC):
    """Base class for all formatter."""
    
    _type: str = None
    _data_input_id: str = None
    _options: Dict = {}
    _project_configuration: Dict
    _original_project_configuration: Dict

    def _normalize_task(self, task: Dict):
        """Make form data behaves the same that iat data"""

        if self._data_input_id:
            return {
                **task,
                "resources" : task.get("data",{}).get("body", {}).get(self._data_input_id, {}).get("resources", []),
                "data": {
                    **task.get("data",{}),
                    "body" : task.get("data",{}).get("body", {}).get(self._data_input_id, {})
                } 
            }
        else:
            return task
    
    ### Configuration
    def _init_configuration(self, project_configuration : Dict, options: Dict, compatibility: Dict):

        self._options = options or {}
        self._original_project_configuration = project_configuration

        project_type = project_configuration['projectType']
        

        # If not compatibility list provided, everything is compatible
        if "compatible_types" not in compatibility:
            self._project_configuration = project_configuration
            return

        # Prepare compatibility
        compatible_projects = [t for t in compatibility["compatible_types"] if t[0:5] != "form."]
        compatible_form_inputs = [t[5:] for t in compatibility["compatible_types"] if t[0:5] == "form."]
        
        if project_type in compatible_projects:
            self._project_configuration = project_configuration
            self._type = project_type
            return
        
        if project_type == "form" and len(compatible_form_inputs) > 0:
            inputs_layout = project_configuration.get('metadata', {}).get('inputsLayout', None)
            
            # If a specific input_id is spotted
            if options.get("input_id", None):
                inputs_for_id = extract_inputs_by_id_from_layout(inputs_layout, options["input_id"])
                input_for_id = inputs_for_id[0] if len(inputs_for_id) > 0 else None
                if not input_for_id or input_for_id["type"] not in compatible_form_inputs:
                    raise Exception('No compatible input found for input_id {} (compatible inputs: {})'.format(options["input_id"], ", ".join(compatible_form_inputs)))
                
                self._data_input_id =  input_for_id['id']
                self._project_configuration = input_for_id['config']
                self._type = input_for_id['type']
                return
            else: # Auto select input if there is only one compatible
                compatible_inputs = extract_inputs_by_types_from_layout(inputs_layout, IAT_INPUT_TYPES)

                if len(compatible_inputs) == 0:
                    raise Exception('No compatible input found (compatible inputs: {})'.format(", ".join(compatible_form_inputs)))
                if len(compatible_inputs) > 1:
                    raise Exception('input_id option is required to select the form input')

                self._data_input_id =  compatible_inputs[0]['id']
                self._project_configuration = compatible_inputs[0]['config']
                self._type =  compatible_inputs[0]['type']
                return


        raise Exception('Project type not compatible')

    def is_iat_project_type(self):
        project_type = self._original_project_configuration['projectType']
        return project_type[0:4] == 'iat-'

    def is_form_project_type(self):
        project_type = self._original_project_configuration['projectType']
        return project_type == 'form'
    
    def get_tool_iat_input(self, project_configuration: Dict = None) -> List:
        if not self.is_form_project_type():
            return []
        if project_configuration == None:
            project_configuration = self._original_project_configuration
        inputs_layout = project_configuration.get('metadata', {}).get('inputsLayout', None)
        if not inputs_layout:
            return []
        
        return extract_inputs_by_types_from_layout(inputs_layout, IAT_INPUT_TYPES)



        

    @abstractmethod
    def format_tasks(self, tasks: List[Dict]) -> List[Dict]:
        raise NotImplementedError

    @abstractmethod
    def append_task_to_export(self, task: Dict):
        raise NotImplementedError
    
    @abstractmethod
    def complete_export(self, tasks: List[Dict]):
        raise NotImplementedError
    
    @abstractmethod
    def prepare_import(self, tasks: List[Dict]):
        raise NotImplementedError